clear all;
addpath('librobotics');
addpath('particlefilter');

% Load data files
disp('Loading data...');
landmarks = load('data/landmarks.dat');
data = read_data('data/sensors.dat');

% Look for GUI axis object
hAxis = findobj('Type','axes','Tag','axesWorld');

% Initialization
global agent_running;   % global variables not recommended :/
global agent_pause;     % easiest way to get values from optional gui
agent_running = 1;
agent_pause = 0;
env = struct;
env.landmarks = landmarks; %for drawing
odom = [0, 0, 0];
X = [];
mean = [0, 0, 0];

% Main loop
for t = 1:size(data.timestep,2)

    %sense (data from file)
    z = data.timestep(t).sensor;
    u = data.timestep(t).odometry;
    
    %self-localization
    [X, mean] = particle_filter(z, u, X, landmarks);
   
    %plot environment, perceptions and belief
    odom = add_odometry(u, odom);
    env.robot = data.timestep(t).groundtruth; %for groundtruth drawing    
    plot_state(hAxis, t, env, z, X, mean, odom);    

    pause(0.1);

    %gui control
    while(agent_pause == 1)
      pause(0.1);
    end
    if(agent_running == 0)
      if(~isempty(hAxis)), cla(hAxis);, end
      break;
    end
end
agent_running = 0;


