function x_new = sample_motion_model(a, x)
    % Samples new particle positions, based on old positions and odometry.
    %
    % a: the movement of the robot (translation x,y and rotation)
    % x: set of old particles

    % Noise parameters
    noise = [0.5 0.3 0.15 0.2];

    % Convert odometry pose to r1,t,r2
    u = struct;
    u.r1 = atan2(a(2), a(1));
    u.t =  sqrt(a(2)^2 + a(1)^2);
    u.r2 = a(3) - u.r1;

    % Particle count
    pc = size(x, 1);

    % Compute normal distributed noise
    O = repmat([u.r1, u.t, u.r2], pc, 1);
    M = zeros(pc, 3);
    S = repmat([ ...
        max(0.00001, noise(1) * abs(u.r1) + noise(2) * u.t) ...
        max(0.00001, noise(3) * u.t  + noise(4) * (abs(u.r1) + abs(u.r2))) ...
        max(0.00001, noise(1) * abs(u.r2) + noise(2) * u.t) ...
        ], pc, 1);
    N = M + randn(pc, 3) .* S;

    % Add noise to the motion for every particle
    odom = O + N;

    % Compute new particle positions
    x_new = x + [ ...
        odom(:, 2) .* cos(x(:, 3) + odom(:, 1)), ...
        odom(:, 2) .* sin(x(:, 3) + odom(:, 1)), ...
        odom(:, 1) + odom(:, 3) ...
        ];
end

