function plot_state(hTarget, timestep, env, percepts, particles, mean, odomPose)
    % Plots the state of the environment and the particles.
    %
    % Displays the robot, landmarks and the particles.
    %
    % hTarget: axis handle to plot on a GUI
    % timestep: current step in the filtering process
    % env: robot and landmarks
    % percepts: seen landmarks
    % particles: current set of particles
    % mean: current belief of the robot pose
    % odomPose: result pose of adding odometry measurements

    % If target handle was given, plot there. Otherwise create figure.
    if(~isempty(hTarget))
      axis(hTarget);
      cla(hTarget)
    else
      hTarget = findobj('type','figure','name','agentFigure');
      if(isempty(hTarget))
        hTarget = figure('name','agentFigure');
      end
      clf(hTarget);
    end
    
    hold on;
    grid('on')


    % Environment
    drawrobot(env.robot, 'r', 3, 0.3, 0.3);
    plot(env.landmarks(:,1), env.landmarks(:,2), 'or');
    
    % Pose from odometry only
    drawrobot(odomPose, 'g', 3, 0.3, 0.3);
    % Percepts
    for i = 1:size(percepts,1)
      zx = env.robot(1) + percepts(i,2) * cos(percepts(i,3) + env.robot(3));
      zy = env.robot(2) + percepts(i,2) * sin(percepts(i,3) + env.robot(3));
      plot([env.robot(1), zx], [env.robot(2), zy], 'g');
    end

    % Particles and mean
    if ~isempty(particles)
      h = plot(particles(:, 1), particles(:, 2), '.b');
      set(h, 'Markersize', 3);
    end
    if ~isempty(mean)
      drawrobot(mean, 'b', 3, 0.3, 0.3); 
    end

    xlim([-7, 7])
    ylim([-5, 5])

    %file output
    %filename = sprintf('../plots/pf_%03d.png', timestep);
    %print(filename, '-dpng');

    hold off
end
