function data = read_data(filename)
    % Reads the odometry and sensor readings from a file.
    %
    % The data is returned in a structure where the u_t and z_t are stored
    % within a single entry. The i-th pair of odometry and measurements are
    % accessed as follows
    % data.timestep(i)
    % this returns a structure of the following type
    % {
    %   groundtruth,
    %   odometry,
    %   sensor
    % }
    % where "odometry" is the motion in x z and rotation, and sensor
    % is a list of sensor readings with id, range, bearing as values.
    input = fopen(filename);

    data = struct;
    data.timestep.sensor = [];
    first = 1;

    groundtruth = [];
    odom = [];
    sensor = [];

    while(~feof(input))
        line = fgetl(input);
        %arr = char(strsplit(line, ' '));
        arr = {};
        while(~isempty(line))
          [arr{end+1}, line] = strtok(line, ' ');
        end
        type = deblank(arr(1));

        if(strcmp(type, 'ROBOT_AND_ODOMETRY') == 1)
            if(first == 0)
                data.timestep(end+1).groundtruth = groundtruth;
                data.timestep(end).odometry = odom;
                data.timestep(end).sensor = sensor;
                groundtruth = [];
                odom = [];
                sensor = [];
            end
            first = 0;
            groundtruth = [str2num(cell2mat(arr(2))), str2num(cell2mat(arr(3))), str2num(cell2mat(arr(4)))];
            odom = [str2num(cell2mat(arr(5))), str2num(cell2mat(arr(6))), str2num(cell2mat(arr(7)))];
        elseif(strcmp(type, 'SENSOR') == 1)
            reading = [str2num(cell2mat(arr(2))), str2num(cell2mat(arr(3))), str2num(cell2mat(arr(4)))];
            if isempty(sensor)
              sensor = reading;
            else
              sensor(end+1,:) = reading;
            end
        end

    end
    data.timestep = data.timestep(2:end);

    fclose(input);
end
